require 'uu/os/security/session'
require 'uu/os/uesuri_builder'
require 'uu/os/use_case_log'
require 'uu/os/logger'

module UU
  module OS
    module VUC

      # Helper for writing into Use Case log.
      # @private
      class UseCaseLogWriter # :nodoc:

        @@msg_queue = Queue.new

        @@semaphore = Mutex.new

        @@worker = nil

        def self.add_record(log_params = {})
          @@msg_queue << log_params
          initialize_writer
        end

        def self.add_log_record(log_params)
          session = log_params[:session]
          view = log_params[:view]
          ctx = view.context
          uri = ctx.main_entity_uri
          path = ctx.controller_url_path
          app_uri = ctx.application_instance_uri
          record = UU::OS::UseCaseLog::UseCaseLogAddRecord.new
          record.creation_time = log_params[:start]
          record.log_level = log_params[:level]
          record.message = log_params[:message]
          record.node_id = get_node_id
          record.object_uri = uri
          record.app_code = UU::OS::UESURI.new(app_uri).artifact_code if app_uri
          path_parts = path.reverse.chomp('/').reverse.split('/')
          uc_name_parts = path_parts[-2].split(/[\-_\.]/).collect(&:capitalize)
          uc_name_parts << path_parts[-1].split(/[\-_\.]/).collect(&:capitalize)
          record.use_case_name = uc_name_parts.join(' ')
          record.use_case_code = path
          record.use_case_type = log_params[:type]
          record.use_case_duration = ((log_params[:finish] - log_params[:start]) * 1000).round(0)
          record.access_role_uri = ctx.access_role_uri
          record.executive_role_uri = ctx.executive_role_uri
          # Append session information last. May invoke remote call in which case we want to have
          # as much information as possible at least in local log.
          if session
            client_app = session.get_client_app_identity_attributes if session
            record.client_app_code = client_app.app_code if client_app
          end
          UU::OS::Security::Session.send(:store_session, session)
          territory_uri = UU::OS::UESURIBuilder.parse_uesuri(uri).set_object_id(nil).set_object_code(nil).set_artifact_id(nil).set_artifact_code(nil).to_uesuri
          UU::OS::UseCaseLog.add_record(territory_uri, record)
        rescue => e
          # XXX VPH-BT:UU.REQ/20171011_0014 - the authorization exception (UU.OS/E05B01.M01) is not traced by this
          # middleware because the exception is traced on server side along with the message.
          if !(e.kind_of?(UU::OS::REST::ServerException) && e.code == "UU.OS/E05B01.M01")
            logger = UU::OS::Logger.new(UseCaseLogMiddleware)
            logger.error("Write to use case log failed:\n\t#{record}", e)
          end
        ensure
          UU::OS::Security::Session.logout(true)
        end
        private_class_method :add_log_record

        def self.initialize_writer
          return if @@worker && @@worker.alive?
          @@semaphore.synchronize do
            return if @@worker && @@worker.alive?
            @@worker = Thread.new do
              loop do
                add_log_record(@@msg_queue.pop)
              end
            end
          end
        end
        private_class_method :initialize_writer

        def self.get_node_id
          node_id = ENV['UU_CLOUD_NODE_NAME'] || ENV['UU_NODE_ID']
          if node_id.nil? && Object.const_defined?(:Java)
            node_id = Java::JavaLang::System.getProperty('uu.node.id')
          end
          return node_id
        end
        private_class_method :get_node_id

      end

    end
  end
end
